#!/bin/bash
#Description: plugin agent install entry
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2024. All rights reserved.
#History: 2024-8-12 created

current_dir=$(readlink -f "$(dirname "$0")")
source "${current_dir}/const.sh"
source "${current_dir}/utils.sh"
install_path="$1/DevKit"
INSTALL_LOG="${install_path}/logs/porting_install.log"
if [[ -f "${INSTALL_LOG}" ]]; then
    rm -f "${INSTALL_LOG}"
fi
touch "${INSTALL_LOG}"
chmod 600 "${INSTALL_LOG}"
architecture=$(uname -m)


EXIT_FLAG="TRUE" # 记录exit_clean状态防止进入死循环
trap "agent_exit_clean" EXIT

# 检查是否有安装残留并清理
function clean_residue()
{
    echo "Clean porting residue"
    x2openEuler_deploy "${INSTALL_LOG}" "uninstall"
    delete_sudo_tools "${INSTALL_LOG}"
    local porting_install_path="${install_path}/devkitplugins/porting"
    if [[ -d "${porting_install_path}" ]]; then
        rm -rf "${porting_install_path}"
    fi
    echo "Clean porting residue success"
}

function agent_exit_clean()
{
    exit_code="$?"
    if [[ ${exit_code} -ne 0 ]] && [[ ${EXIT_FLAG} == "TRUE" ]] ; then
        EXIT_FLAG="FALSE"
        clean_residue
        exit 1
    fi
}

function init_variable()
{
    if [[ ! -f "${CONFIG_FILE}" ]]; then
        echo_and_log "There is no configure file(${CONFIG_FILE}), please check!" "${ERROR}"
        exit "${ERROR}"
    fi
    INSTALL_PLUGIN_PATH="${install_path}/devkitplugins"
}

function check_kunpeng()
{
    if [[ "${architecture}" != "aarch64" ]]; then
        echo_and_log "\nFailed to install the Porting Plugin. Only the Kunpeng platform is supported." "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    else
        if ! is_supported_arm; then
            echo_and_log "\nFailed to install the Porting Plugin. Only the Kunpeng platform is supported." "${ERROR}" "${INSTALL_LOG}"
            exit "${ERROR}"
        fi
    fi
}

function is_supported_arm()
{
    current_cpu_id="unknown"
    if [[ -f "${CPU_ID_FILE}" ]]; then
        current_cpu_id=$(cat "${CPU_ID_FILE}")
    fi
    for value in "${SUPPORTED_ARM[@]}"
    do
        if [[ "${current_cpu_id}" == "${value}" ]]; then
            return "${SUCCESS}"
        fi
    done
    return "${ERROR}"
}

function installed_check()
{
    if [[ -f "${SUDO_FILE}" ]]; then
        echo_and_log "Porting agent has already been deployed!" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    fi
}

function agent_libcheck()
{
    bash "${current_dir}/agent_env_check.sh" "${install_path}" "agent"
    if [[ "$?" -ne "${SUCCESS}" ]]; then
        echo_and_log "Check install Dependent Packages Failed, Stop installing porting agent" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    else
        echo_and_log "Check install Dependent Packages Success" "${SUCCESS}" "${INSTALL_LOG}"
    fi
}

function clean_devkitclient_process()
{
    devkitclient_pid=$(ps -ef | grep "tools/rpc_cluster/client/bin/devkitclient" | grep -v grep | awk '{print $2}')
    if [[ -n "${devkitclient_pid}" ]]; then
        kill -15 "${devkitclient_pid}"
        if [ $? -eq 1 ];then
            echo_and_log "The devkitclient process still exists. " "${ERROR}" "${INSTALL_LOG}"
            exit "${ERROR}"
        fi
    fi
}

function change_file_path_permission()
{
    find "${INSTALL_PLUGIN_PATH}/porting/deploy" -name "*\.sh" -exec chown root:root {} \; >> /dev/null 2>&1
    find "${INSTALL_PLUGIN_PATH}/porting/deploy" -name "*\.sh" -exec chmod 500 {} \; >> /dev/null 2>&1
    if [[ -d "${INSTALL_PLUGIN_PATH}/porting" ]]; then
        find "${INSTALL_PLUGIN_PATH}/porting" -type d -exec chmod 700 {} \; >> /dev/null 2>&1
        find "${INSTALL_PLUGIN_PATH}/porting" -type f -exec chmod 400 {} \; >> /dev/null 2>&1
        chmod 755 "${INSTALL_PLUGIN_PATH}/porting"
        chown root:root "${INSTALL_PLUGIN_PATH}/porting"
    fi

    if [[ -d "${INSTALL_PLUGIN_PATH}/porting/cmd/bin" ]]; then
        chown -R root:root "${INSTALL_PLUGIN_PATH}"/porting/cmd/bin/*
        find "${INSTALL_PLUGIN_PATH}/porting/cmd/bin" -type d -exec chmod 705 {} \; >> /dev/null 2>&1
        chown root:root "${INSTALL_PLUGIN_PATH}/porting/cmd/bin"
        chmod 705 "${INSTALL_PLUGIN_PATH}/porting/cmd/bin"
        chown root:root "${INSTALL_PLUGIN_PATH}/porting/cmd"
        chmod 705 "${INSTALL_PLUGIN_PATH}/porting/cmd"
    fi

    local bin_files=(
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/sys-mig"
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/middlewaremig"
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/osmig"
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/exceptionhandler"
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/resources/dbmp_resources/dbmp/bin/dbmp"
    )

    for bin_file in "${bin_files[@]}"; do
        if [[ -f "${bin_file}" ]]; then
            chmod 505 "${bin_file}"
        fi
    done

    local x2openEuler_deploy_path="${INSTALL_PLUGIN_PATH}/porting/cmd/x2openEuler_install/deploy-x2openEuler"
    if [[ -f "${x2openEuler_deploy_path}" ]]; then
        chown root:root "${x2openEuler_deploy_path}"
        chmod 500 "${x2openEuler_deploy_path}"
    fi
    local x2openEuler_rpm_path="${INSTALL_PLUGIN_PATH}/porting/cmd/x2openEuler_install/rpm"
    if [[ -d "${x2openEuler_rpm_path}" ]]; then
        find "${x2openEuler_rpm_path}" -name "*\.rpm" -exec chown root:root {} \; >> /dev/null 2>&1
        find "${x2openEuler_rpm_path}" -name "*\.rpm" -exec chmod 500 {} \; >> /dev/null 2>&1
    fi
}

function add_sudoers()
{
    local install_path=$1
    local install_log=$2

    parameter_list=(
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/sys-mig"
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/middlewaremig"
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/osmig"
        "${INSTALL_PLUGIN_PATH}/porting/cmd/bin/exceptionhandler"
    )
    parameter_list_len=${#parameter_list[@]}
    for ((i=0;i<${parameter_list_len};i++)); do
        local parameter=${parameter_list[i]}
        local cmd_name
        cmd_name=$(echo ${parameter} | cut -d " " -f 1)
        add_devkit_plugin_sudo "${cmd_name}" "${parameter}" "${install_log}" "${SUDO_FILE}"
    done
    echo_and_log "Add porting sudoers success" "${SUCCESS}" "${install_log}"
}

function add_devkit_plugin_sudo()
{
    local command="$1"
    local command_path="$2"
    local install_log="$3"
    local sudo_file="$4"
    # 跳过空提权命令
    if [[ -z "${command}" ]]; then
        return
    fi
    # 跳过不存在的提权命令
    if [[ ! -e "${command}" ]]; then
        echo_and_log  "Not exits command, skip: ${command}" "${DOING}" "${install_log}"
        return
    fi

    if [[ ! -f "${sudo_file}" ]]; then
        touch "${sudo_file}"
    fi
    chmod 440 "${sudo_file}"

    if [[ ! -z "${command_path}" ]]; then
        # 空文件sed 追加不进去 sed 根据行号追加内容
        echo "${DEVKIT_USER}    ALL=(root)      NOPASSWD:""${command_path}" >> "${sudo_file}"
    fi
}

function create_log_dir()
{
    local log_dir="${install_path}/logs/"
    if [[ ! -d "${log_dir}" ]]; then
        mkdir -p "${log_dir}"
        chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${log_dir}"
        chmod 700 "${log_dir}"
    fi
}

function main()
{
    init_variable
    check_kunpeng "${INSTALL_LOG}"
    create_log_dir
    installed_check
    echo_and_log "Installing porting agent" "${DOING}" "${INSTALL_LOG}"
    agent_libcheck
    clean_devkitclient_process
    change_file_path_permission
    add_sudoers "${install_path}" "${INSTALL_LOG}"
    # 所有安装完成后，更改日志文件权限为只读
    chmod 400 "${INSTALL_LOG}"
    echo_and_log "Installing porting agent success" "${SUCCESS}" "${INSTALL_LOG}"
}

main $@